@@ -30,7 +30,7 @@ int dvb_desc_t2_delivery_init(struct dvb_v5_fe_parms *parms,
 	struct dvb_desc_t2_delivery *d = desc;
 	unsigned char *p = (unsigned char *) buf;
 	size_t desc_len = ext->length - 1, len, len2;
-	int i;
+	int i, n, pos = 0, subcel_length;
 
 	len = offsetof(struct dvb_desc_t2_delivery, bitfield);
 	len2 = offsetof(struct dvb_desc_t2_delivery, centre_frequency);
@@ -40,53 +40,76 @@ int dvb_desc_t2_delivery_init(struct dvb_v5_fe_parms *parms,
 		return -1;
 	}
 	if (desc_len < len2) {
-		memcpy(p, buf, len);
+		memcpy(d, buf, len);
 		bswap16(d->system_id);
 
-		if (desc_len != len)
-			dvb_logwarn("T2 delivery descriptor is truncated");
+		/* It is valid to have length == 4 */
+		if (desc_len == len)
+			return 0;
 
+		dvb_logwarn("T2 delivery descriptor is truncated");
 		return -2;
 	}
-	memcpy(p, buf, len2);
+	memcpy(d, buf, len2);
+	bswap16(d->system_id);
+	bswap16(d->bitfield);
 	p += len2;
 
-	len = desc_len - (p - buf);
-	memcpy(&d->centre_frequency, p, len);
-	p += len;
+	while (desc_len - (p - buf)) {
+		if (desc_len - (p - buf) < sizeof(uint16_t)) {
+			dvb_logwarn("T2 delivery descriptor is truncated");
+			return -2;
+		}
 
-	if (d->tfs_flag)
-		d->frequency_loop_length = 1;
-	else {
-		d->frequency_loop_length = *p;
+
+		/* Discard cell ID */
+		p += sizeof(uint16_t);
+
+		if (d->tfs_flag) {
+			n = *p;
+			p++;
+		}
+		else
+			n = 1;
+
+		d->frequency_loop_length += n;
+		d->centre_frequency = realloc(d->centre_frequency,
+					      d->frequency_loop_length * sizeof(*d->centre_frequency));
+		if (!d->centre_frequency) {
+			dvb_logerr("%s: out of memory", __func__);
+			return -3;
+		}
+
+		memcpy(&d->centre_frequency[pos], p, sizeof(*d->centre_frequency) * n);
+		p += sizeof(*d->centre_frequency) * n;
+
+		for (i = 0; i < n; i++) {
+			bswap32(d->centre_frequency[pos]);
+			pos++;
+		}
+
+		/* Handle subcel frequency table */
+		subcel_length = *p;
 		p++;
+		for (i = 0; i < subcel_length; i++) {
+			if (desc_len - (p - buf) < sizeof(uint8_t) + sizeof(uint32_t)) {
+				dvb_logwarn("T2 delivery descriptor is truncated");
+				return -2;
+			}
+			p++;	// Ignore subcell ID
+
+			// Add transposer_frequency at centre_frequency table
+			d->frequency_loop_length++;
+			d->centre_frequency = realloc(d->centre_frequency,
+						      d->frequency_loop_length * sizeof(*d->centre_frequency));
+			memcpy(&d->centre_frequency[pos], p, sizeof(*d->centre_frequency));
+			bswap32(d->centre_frequency[pos]);
+			pos++;
+
+			p += sizeof(*d->centre_frequency);
+		}
 	}
 
-	d->centre_frequency = calloc(d->frequency_loop_length,
-				     sizeof(*d->centre_frequency));
-	if (!d->centre_frequency) {
-		dvb_logerr("%s: out of memory", __func__);
-		return -3;
-	}
-
-	memcpy(d->centre_frequency, p, sizeof(*d->centre_frequency) * d->frequency_loop_length);
-	p += sizeof(*d->centre_frequency) * d->frequency_loop_length;
-
-	for (i = 0; i < d->frequency_loop_length; i++)
-		bswap32(d->centre_frequency[i]);
-
-	d->subcel_info_loop_length = *p;
-	p++;
-
-	d->subcell = calloc(d->subcel_info_loop_length, sizeof(*d->subcell));
-	if (!d->subcell) {
-		dvb_logerr("%s: out of memory", __func__);
-		return -4;
-	}
-	memcpy(d->subcell, p, sizeof(*d->subcell) * d->subcel_info_loop_length);
-
-	for (i = 0; i < d->subcel_info_loop_length; i++)
-		bswap16(d->subcell[i].transposer_frequency);
 	return 0;
 }
 
@@ -105,19 +128,16 @@ void dvb_desc_t2_delivery_print(struct dvb_v5_fe_parms *parms,
 
 	dvb_loginfo("|           tfs_flag                  %d", d->tfs_flag);
 	dvb_loginfo("|           other_frequency_flag      %d", d->other_frequency_flag);
-	dvb_loginfo("|           transmission_mode         %d", d->transmission_mode);
-	dvb_loginfo("|           guard_interval            %d", d->guard_interval);
+	dvb_loginfo("|           transmission_mode         %s (%d)",
+		    fe_transmission_mode_name[dvbt2_transmission_mode[d->transmission_mode]], d->transmission_mode);
+	dvb_loginfo("|           guard_interval            %s (%d)",
+		    fe_guard_interval_name[dvbt2_interval[d->guard_interval]], d->guard_interval );
 	dvb_loginfo("|           reserved                  %d", d->reserved);
-	dvb_loginfo("|           bandwidth                 %d", d->bandwidth);
-	dvb_loginfo("|           SISO MISO                 %d", d->SISO_MISO);
+	dvb_loginfo("|           bandwidth                 %d", dvbt2_bw[d->bandwidth]);
+	dvb_loginfo("|           SISO MISO                 %s", siso_miso[d->SISO_MISO]);
 
 	for (i = 0; i < d->frequency_loop_length; i++)
 		dvb_loginfo("|           centre frequency[%d]   %d", i, d->centre_frequency[i]);
-
-	for (i = 0; i < d->subcel_info_loop_length; i++) {
-		dvb_loginfo("|           cell_id_extension[%d]  %d", i, d->subcell[i].cell_id_extension);
-		dvb_loginfo("|           transposer frequency   %d", d->subcell[i].transposer_frequency);
-	}
 }
 
 void dvb_desc_t2_delivery_free(const void *desc)
@@ -159,3 +179,8 @@ const unsigned dvbt2_transmission_mode[] = {
 	[5] = TRANSMISSION_MODE_32K,
 	[6 ...7] = TRANSMISSION_MODE_AUTO,	/* Reserved */
 };
+const char *siso_miso[4] = {
+	[0] = "SISO",
+	[1] = "MISO",
+	[2 ...3] = "reserved",
+};
