@@ -103,12 +103,26 @@ void common_usage(void)
 	       );
 }
 
+static const char *prefixes[] = {
+	"video",
+	"radio",
+	"vbi",
+	"swradio",
+	"v4l-subdev",
+	NULL
+};
+
 static bool is_v4l_dev(const char *name)
 {
-	return !memcmp(name, "video", 5) ||
-		!memcmp(name, "radio", 5) ||
-		!memcmp(name, "vbi", 3) ||
-		!memcmp(name, "v4l-subdev", 10);
+	for (unsigned i = 0; prefixes[i]; i++) {
+		unsigned l = strlen(prefixes[i]);
+
+		if (!memcmp(name, prefixes[i], l)) {
+			if (isdigit(name[l]))
+				return true;
+		}
+	}
+	return false;
 }
 
 static int calc_node_val(const char *s)
@@ -116,12 +130,17 @@ static int calc_node_val(const char *s)
 	int n = 0;
 
 	s = strrchr(s, '/') + 1;
-	if (!memcmp(s, "video", 5)) n = 0;
-	else if (!memcmp(s, "radio", 5)) n = 0x100;
-	else if (!memcmp(s, "vbi", 3)) n = 0x200;
-	else if (!memcmp(s, "v4l-subdev", 10)) n = 0x300;
-	n += atol(s + (n >= 0x200 ? 3 : 5));
-	return n;
+
+	for (unsigned i = 0; prefixes[i]; i++) {
+		unsigned l = strlen(prefixes[i]);
+
+		if (!memcmp(s, prefixes[i], l)) {
+			n = i << 8;
+			n += atol(s + l);
+			return n;
+		}
+	}
+	return 0;
 }
 
 static bool sort_on_device_name(const std::string &s1, const std::string &s2)
@@ -191,8 +210,10 @@ static void list_devices()
 
 		if (fd < 0)
 			continue;
-		doioctl(fd, VIDIOC_QUERYCAP, &vcap);
+		int err = ioctl(fd, VIDIOC_QUERYCAP, &vcap);
 		close(fd);
+		if (err)
+			continue;
 		bus_info = (const char *)vcap.bus_info;
 		if (cards[bus_info].empty())
 			cards[bus_info] += std::string((char *)vcap.card) + " (" + bus_info + "):\n";
@@ -422,12 +443,12 @@ static int print_control(int fd, struct v4l2_query_ext_ctrl &qctrl, int show_men
 		print_qctrl(fd, &qctrl, NULL, show_menus);
 		return 1;
 	}
-	ctrls.ctrl_class = V4L2_CTRL_ID2CLASS(qctrl.id);
+	ctrls.which = V4L2_CTRL_ID2WHICH(qctrl.id);
 	ctrls.count = 1;
 	ctrls.controls = &ext_ctrl;
 	if (qctrl.type == V4L2_CTRL_TYPE_INTEGER64 ||
 	    qctrl.type == V4L2_CTRL_TYPE_STRING ||
-	    (V4L2_CTRL_ID2CLASS(qctrl.id) != V4L2_CTRL_CLASS_USER &&
+	    (V4L2_CTRL_ID2WHICH(qctrl.id) != V4L2_CTRL_CLASS_USER &&
 	     qctrl.id < V4L2_CID_PRIVATE_BASE)) {
 		if (qctrl.type == V4L2_CTRL_TYPE_STRING) {
 		    ext_ctrl.size = qctrl.maximum + 1;
@@ -862,7 +883,7 @@ void common_set(int fd)
 					use_ext_ctrls = true;
 				ctrl.value = strtol(iter->second.c_str(), NULL, 0);
 			}
-			class2ctrls[V4L2_CTRL_ID2CLASS(ctrl.id)].push_back(ctrl);
+			class2ctrls[V4L2_CTRL_ID2WHICH(ctrl.id)].push_back(ctrl);
 		}
 		for (class2ctrls_map::iterator iter = class2ctrls.begin();
 				iter != class2ctrls.end(); ++iter) {
@@ -883,7 +904,7 @@ void common_set(int fd)
 				continue;
 			}
 			if (iter->second.size()) {
-				ctrls.ctrl_class = iter->first;
+				ctrls.which = iter->first;
 				ctrls.count = iter->second.size();
 				ctrls.controls = &iter->second[0];
 				if (doioctl(fd, VIDIOC_S_EXT_CTRLS, &ctrls)) {
@@ -989,7 +1010,7 @@ void common_get(int fd)
 			}
 			if (V4L2_CTRL_DRIVER_PRIV(ctrl.id))
 				use_ext_ctrls = true;
-			class2ctrls[V4L2_CTRL_ID2CLASS(ctrl.id)].push_back(ctrl);
+			class2ctrls[V4L2_CTRL_ID2WHICH(ctrl.id)].push_back(ctrl);
 		}
 		for (class2ctrls_map::iterator iter = class2ctrls.begin();
 				iter != class2ctrls.end(); ++iter) {
@@ -1006,7 +1027,7 @@ void common_get(int fd)
 				continue;
 			}
 			if (iter->second.size()) {
-				ctrls.ctrl_class = iter->first;
+				ctrls.which = iter->first;
 				ctrls.count = iter->second.size();
 				ctrls.controls = &iter->second[0];
 				doioctl(fd, VIDIOC_G_EXT_CTRLS, &ctrls);
